from __future__ import absolute_import
from decimal import Decimal
import re
import json

from ..models import PaymentApiToken
from ..exceptions import ApiException
from ..utils import LoggerUtil
from .base_request import BaseRequest
from .payment_api import PaymentApi
from ..api_client import ApiClient
from ..models import PaymentTokenRequest
from ..utils import TimeUtil
from ..utils import ValidationUtil
from ..utils.encryption_util import EncryptionUtil
from ..models import Fee
from ..models import Payment
from ..models import Error


class Payment(BaseRequest, object):
    logger = LoggerUtil.setup_logger(__name__)
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'customer_account_reference': 'str',
        'digi_sign': 'str',
        'custom_fields': 'dict',
        'payment_token': 'PaymentApiToken',
        'amount': 'str',
        'fee_amount' : 'str'
    }

    attribute_map = {
        'customer_account_reference': 'customer_account_reference',
        'digi_sign': 'digi_sign',
        'custom_fields': 'custom_fields',
        'payment_token': 'payment_token',
        'amount': 'amount',
        'fee_amount' : 'fee_amount'
    }

    def __init__(self, customer_account_reference=None, amount=None, digi_sign = None, payment_token=None, custom_fields=None, fee_amount=None):

        self._customer_account_reference = None
        self._amount = None
        self._digi_sign = None
        self._payment_token = None
        self.__custom_fields = None
        self._fee_amount = None

        if customer_account_reference is not None:
            self._customer_account_reference = customer_account_reference
        if amount is not None:
            self.amount = amount
        if fee_amount is not None:
            self.fee_amount = fee_amount
        self.digi_sign = digi_sign
        self.payment_token = payment_token
        self.custom_fields = custom_fields

    @property
    def payment_token(self):
        """
        Gets the payment_token of this Payment.

        :return: The payment_token of this Payment.
        :rtype: str
        """
        return self._payment_token

    @payment_token.setter
    def payment_token(self, payment_token):
        """
        Sets the payment_token of this Payment.

        :param payment_token: The payment_token of this Payment.
        :type: str
        """
        self._payment_token = payment_token

    @property
    def amount(self):
        """
        Gets the amount of this Payment.
        Amount fields are of format N [9, 2] with . as decimal separator         (for e.g. 100.20).

        :return: The amount of this Payment.
        :rtype: str
        """
        return self._amount

    @amount.setter
    def amount(self, amount):
        """
        Sets the amount of this Payment.
        Amount fields are of format N [9, 2] with . as decimal separator         (for e.g. 100.20).

        :param amount: The amount of this Payment.
        :type: str
        """
        self._amount = amount

    @property
    def customer_account_reference(self):
        return self._customer_account_reference

    @customer_account_reference.setter
    def customer_account_reference(self, customer_account_reference):
        self._customer_account_reference = customer_account_reference

    @property
    def digi_sign(self):
        return self._digi_sign

    @digi_sign.setter
    def digi_sign(self, digi_sign):
        self._digi_sign = digi_sign

    @property
    def custom_fields(self):
        return self._custom_fields

    @custom_fields.setter
    def custom_fields(self, custom_fields):
        self._custom_fields = custom_fields

    @property
    def fee_amount(self):
        return self._fee_amount

    @fee_amount.setter
    def fee_amount(self, fee_amount):
        self._fee_amount = fee_amount
    
    def with_custom_fields(self, custom_fields):
        self._custom_fields = custom_fields
        return self

    def with_token(self, data, digi_sign):
        if isinstance(data, PaymentApiToken):
            self._payment_token = data
        else:
            self._payment_token = PaymentApiToken(data, self._amount)

        self._digi_sign = digi_sign
        return self

    def for_partner(self, client_id, signature_key):
        self.client_id = client_id
        self.signature_key = signature_key
        return self

    def for_client(self, client_id, signature_key):
        self.client_id = client_id
        self.signature_key = signature_key
        return self

    def with_fee(self, fee_amount):
        self._fee_amount = fee_amount
        return self

    def confirm(self):
        Payment.logger.info('confirm_payment')
        try:

            if ValidationUtil.is_valid_payment_token_request(self):
                
                decrypted_token = EncryptionUtil.decrypt(self._payment_token.token)
                
                is_valid_sign = EncryptionUtil.verify(decrypted_token, self.digi_sign)
                
                if is_valid_sign:
                    self._payment_token.token = EncryptionUtil.encrypt(decrypted_token)
                    self.digi_sign = EncryptionUtil.sign(decrypted_token)
                else:
                    raise ApiException(
                        status=0,
                        reason="Invalid token_id or digi_sign"
                    )
                
                tokenizedAmount = float(self._payment_token.tokenized_amount)
                
                #rounding off the amount to two decimal places
                sign = 1 if (tokenizedAmount >= 0) else -1;
                roundedAmount = (round((tokenizedAmount*pow(10,2)) + (sign*0.001)) / pow(10,2));

                Payment.logger.info('Rounded amount:: '+str(roundedAmount))
                Payment.logger.info('fee is: '+str(self._fee_amount))
                self._payment_token.tokenized_amount = EncryptionUtil.encrypt(str(roundedAmount))
                api_client = ApiClient(client_id=self.client_id, signature_key=self.signature_key)
                payment_token_request = PaymentTokenRequest(self.customer_account_reference, self.custom_fields, self._payment_token, self._fee_amount)
                payment_api = PaymentApi(api_client)
                start_time = TimeUtil.get_time_in_milli_seconds()


                Payment.logger.info('OBJ '+str(payment_token_request))
                payment_response = payment_api.payment_token(payment_token_request=payment_token_request,
                                                                   client_id=self.client_id,
                                                                   digi_sign=self.digi_sign)
                end_time = TimeUtil.get_time_in_milli_seconds()
                Payment.logger.info('Time taken for confirm_payment: {}'.format(end_time - start_time))
                return payment_response
            else:
                raise ApiException(
                    status=400,
                    reason='Invalid confirm Payment request.'
                )
        except BaseException as e:
            try:

                Payment.logger.error("exception occured: {}".format(e))
                error_python_object = json.loads(format(e))['errors']
                error_message = error_python_object[0]['message']
                error_code = error_python_object[0]['code']
                error_field = error_python_object[0]['field']
                payment = Payment()
                errors = Error(error_code,error_message,error_field)
                payment.errors = errors
                return payment
            except BaseException as ee:
                raise ApiException(
                status=400,
                #reason= format(ee)
                reason='Failed to confirm_payment_request. {}'.format(e)
                )